##############################################################################

"""XPath Expression Handler

   Copyright 2001, Burning Tiger Technologies
   Written by Dethe Elza

"""

from xml import xpath 
from xml.dom import Node
from xml.xpath.Context import Context
from types import StringType, FloatType, ClassType
from Products.PageTemplates.TALES import TALESError
from Ft.Lib import FtException
from Ft.Lib.pDomlette import PyExpatReader
import cgi
try:
  #from StringIO import StringIO
  from Products.ParsedXML import ExtraDOM
except:
  pass
  

def install():
  from Products.PageTemplates.Expressions import getEngine
  engine = getEngine()
  try:
    engine.registerType("xpath", XPathExpr)
  except:
    pass # expect to fail on refresh, put better test here...

class XPathExpr:
  render_flag = 1 # do we return objects or string?
  context = None # context is target doc for xpath
  expr = None # the parsed xpath expression

  def __init__(self, name, expr, engine):
    self.name = name
    self.orig_expr = expr
    self.ns = {} # xml namespaces for xpath
    self._parse(expr, engine)

  def _parse(self, expr_string, engine):
    ''' Break original expression string into several variables: 
      render_flag - should we return objects or a string?
      context node - hopefully the root node of document to apply path to
      context namespace - the dictionary of namespaces to use in xpath evaluation
      expr - the actual xpath expression to use

      NOTE: We go to some trouble to handle a lot of variation in the
      expression.  We loop through it several times to pull out the various
      pieces.  We limit all splits to 1 because the characters we split on
      can occur in the xpath expression itself.  We use 'continue' in the 
      loop as soon as we've found data, because with the exception that the 
      actual expression must be last, the rest can occur in any order.  Note
      that whitespace within each piece is illegal.

      TODO:
      * Allow python dictionarys to be passed in as namespaces, for reuse
    '''
    expr_string = expr_string.strip()
    #index = 0
    while 1:
      # check for structure
      if (self.render_flag) and expr_string.startswith('structure'):
        discard, expr_string = expr_string.split(None, 1)
        self.render_flag = 0
	continue
      # check for namespace
      if expr_string.startswith('xmlns:'):
        ns_string, expr_string = expr_string.split(None, 1)
	discard, ns_string = ns_string.split(':', 1)
	prefix, ns = ns_string.split('=', 1)
	self.ns[prefix] = ns
	continue
      # check for context
      if not self.context:
        context, expr_string = expr_string.split(None, 1)
        self.context = engine.compile(context)
        continue
      # expr remains
      self.expr = expr_string
      break

  def _xmlParse(self, xml_string):
    reader = PyExpatReader()
    doc = reader.fromString(xml_string)
    return doc

  def _getContextNode(self, context_obj):
    # Note: we're going to have to make sure we handle types correctly
    # 1. If it has a root node method, process as XML
    # 2. If it doesn't have a root node, or XML fails, try to get content
    #    and parse as string.
    # 3. If Zope content methods fail, try str(context) and parse as string
    # 4. If all attempts fail, return an error message as string.
    if type(context_obj) is StringType:
      # It's a string, let's try to parse it
      try:
        return self._xmlParse(context_obj)
      except FtException:
        # we're running on a document which isn't well-formed
        raise XPathExprError("_getContextNode", "Parse Exception")
    elif type(context_obj) is ClassType:
        #it's a class, all classes have the same type
        if hasattr(context_obj, 'nodeType'):
          return context_obj # already a node
        try:
          context_obj.getNodeType()
          return context_obj # already a node
        except:
          # not a DOM node, maybe we can turn it into a string?
          raise XPathExprError("_getContextNode",
            "This should be a DOM Node, %s" % self.formatError(context_obj))
    else:
      # We don't recognize this type
      try:
        context_obj.getNodeType()
        return context_obj # already a node
      except:
        pass
      try:
        # try processing it as a string
        # FIXME: This works on ParsedXML, but it would be nicer to treat 
        # those objects as DOM
        #raise XPathExprError('_getContextNode','<pre>' + str(context_obj) + '</pre>')
        return self._xmlParse(str(context_obj))
      except FtException:
        raise XPathExprError("_getContextNode",
           "Unrecognized Context %s" % self.formatError(context_obj))

  def formatError(self, obj):
    retval = ""
    try:
      retval = "value is %s (%s), type is %s, bases=%s" % (obj, 
         dir(obj), type(obj), obj.__bases__)
    except:
      retval = "value is %s (%s), type is %s" % (obj, dir(obj), type(obj))
    return cgi.escape(retval)

  def _hasAttributes(self, node):
    try:
      if node.hasAttributes():
        return 1
    except:
      pass
    try:
      if node.attributes:
        return 1
    except:
      pass
    return 0

  def _hasChildNodes(self, node):
    try:
      if node.hasChildNodes():
        return 1
    except:
      pass
    try:
      if node.childNodes:
        return 1
    except TALESError:
      pass
    return 0

  def __call__(self, econtext):
    xcontext_val = self._getContextNode(self.context(econtext))
    xcontext = Context(xcontext_val, 0, 0, None, self.ns)
    # What the hell are we getting?
    #raise XPathExprError("_call",
    #  "Expr: %s" % self.formatError(xcontext))
    result = xpath.Evaluate(self.expr, context=xcontext)
    if not self.render_flag:
      # Is this really what we want?
      return result
    return self._resultString(result)

  def _formatNode(self, node):
    prefix = ""
    if node.prefix:
      prefix = node.prefix + ':'
    if node.nodeType == Node.ATTRIBUTE_NODE:
      return ' ' + prefix + node.name + '="' + node.value + '"\n'
    if node.nodeType == Node.TEXT_NODE and node.nodeValue is not None:
      return '\n' + node.nodeValue + '\n'
    if node.nodeType == Node.ELEMENT_NODE:
      out = ""
      out += '<' + prefix + node.nodeName
      if self._hasAttributes(node):
        try:
          for attr in node.attributes:
            out += ' ' + self._formatNode(attr)
        except:
          pass
      if self._hasChildNodes(node):
        out += '>'
        #out += '[children ...]'
        for child in node.childNodes:
          out += self._formatNode(child)
        out += '</' + node.nodeName + '>\n'
      else:
        out += '/>\n'
    return out
        

  def _xmlPrettyPrint(self, objlist):
    out = ""
    if type(objlist) == type([]):
      for node in objlist:
        try:
          # use better printing if available
          out += ExtraDOM.writeStream(node).getValue()
        except:
          out += self._formatNode(node);
    elif type(objlist) == type(0.0):
      out = str(objlist)
    elif type(objlist) == type(""):
      out = objlist
    else:
      raise XPathExprError('_xmlPrettyPrint', 'Unexpected Type ' + str(type(objlist)))
    return out

  def _resultString(self, result):
    # pretty-print a node list, let the rest be
    return self._xmlPrettyPrint(result)

  def __str__(self):
    render = 'false'
    if self.render_flag:
      render = 'true'
    return "<xpath expression name=%s expr=%s context=%s namespace=%s render=%s" % (self.name, self.expr, self.context, self.ns, render)

  def __repr__(self):
    return "&lt;XPathExpr name=%s expr=%s>" % (self.name, self.orig_expr)


class XPathExprError(ValueError):
  def __init__(self, func, info):
    self._info = info
    self._func = func
  def __str__(self):
    return "XPathExpr." + self._func + "(): " + self._info
