import os, sys
execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PageTemplates import XPathExpr

class Container:
  pass

class XPathTests(unittest.TestCase):

  def setup(self):
    # setup some context data
    doc1 = '''<?xml version="1.0"?>
<root ratt1="ratt1" ratt2="ratt2">
  <container>
    <elem eatt1="eatt1" eatt2="eatt2">
      This is the content of first elem
    </elem>
    <elem eatt1="eatt3" eatt2="eatt4">
      This is the content of second elem
    </elem>
  </container>
  <elem eatt1="eatt5" eatt2="eatt6">
    This is the content of third elem
  </elem>
</root>
'''
    doc2 = '''<?xml version="1.0"?>
<root ratt1="ratt1" ratt2="ratt2" xmlns="http://livingcode.ca/freeword">
  <container>
    <elem eatt1="eatt1" eatt2="eatt2">
      This is the content of first elem
    </elem>
    <elem eatt1="eatt3" eatt2="eatt4">
      This is the content of second elem
    </elem>
  </container>
  <elem eatt1="eatt5" eatt2="eatt6">
    This is the content of third elem
  </elem>
</root>
    '''
    self.here = Container()
    self.container = Container()
    self.here.doc_no_ns = doc1
    self.here.doc_ns = doc2
    self.container.doc_no_ns = doc1
    self.container.doc_ns = doc2
    self.expressions = [
          'xpath: structure here.doc_no_ns //elem',
          'xpath:structure container.doc_no_ns //elem/@* ',
          'xpath:container.doc_no_ns //elem',
          'xpath:container.doc_no_ns //elem/@*',
    ]
    self.dom_expressions = [
          'xpath: structure here.dom_doc //elem',
          'xpath:structure here.dom_doc //elem/@* ',
          'xpath:here.dom_doc //elem',
          'xpath:here.dom_doc //elem/@*',
    ]
    self.ns_expressions = [
          'xpath:structure here.doc_ns xmlns:my=http://livingcode.ca/freeword //my:elem',
          'xpath: structure  container.doc_ns xmlns:my=http://livingcode.ca/freeword //my:elem',
          'xpath:container.doc_ns xmlns:my=http://livingcode.ca/freeword //my:elem/@*',
          'xpath:container.doc_ns xmlns:my=http://livingcode.ca/freeword //my:elem/@*',
          'xpath:xmlns:my=http://livingcode.ca/freeword container.doc_ns //my:elem[0]',
          'xpath: here.doc_ns xmlns:my=http://livingcode.ca/freeword structure xmlns:yours=http://burnintiger.com/hobbes 2 + 3',
          'xpath: xmlns:yours=http://burningtiger.com/hobbes structure   here.doc_ns  xmlns:my=http://livingcode.ca/freeword true and false',
          'xpath: here.doc_ns xmlns:my=http://livingcode.ca/freeword xmlns:yours=http://burnintiger.com/hobbes //elem',
          'xpath: xmlns:yours=http://burningtiger.com/hobbes here.doc_ns  xmlns:my=http://livingcode.ca/freeword /*',
        ]

  def testCompile(self):
    '''Test expression compilation'''
    # set up some queries to test
    self.setup()
    e = XPathExpr.getEngine()
    for expr in self.expressions:
      e.compile(expr)
    for expr in self.ns_expressions:
      e.compile(expr)

  def testEval(self):
    '''Test expression evaluation'''
    self.setup()
    e = XPathExpr.getEngine()
    c = e.getContext(self.__dict__)
    for expr in self.expressions:
      print ""
      print 'Evaluating ' + expr
      print c.evaluate(expr)
      print "-_-_-_-_-_-_"

  def testEvalNS(self):
    '''Test expression evaluation'''
    self.setup()
    e = XPathExpr.getEngine()
    c = e.getContext(self.__dict__)
    for expr in self.ns_expressions:
      print ""
      print 'Evaluating ' + expr
      print c.evaluate(expr)
      print "-_-_-_-_-_-_"

  def _parse(self, xml_string):
    '''Parse a string, return the resulting DOM'''
    from Ft.Lib import FtException
    from Ft.Lib.pDomlette import PyExpatReader
    try:
      reader = PyExpatReader()
      doc = reader.fromString(xml_string)
      return doc
    except FtException:
      # we're running on a document which isn't well-formed
      raise XPathExprError("_getContextNode", "Parse Exception on " + str(type(context_obj)))
 
  def testEvalWithDOM(self):
    '''Test our dom passing code'''
    self.setup()
    e = XPathExpr.getEngine()
    d = self._parse(self.here.doc_no_ns)
    self.here.dom_doc = d
    c = e.getContext(self.__dict__)
    for expr in self.dom_expressions:
      print ""
      print 'Evaluating with DOM' + expr
      print c.evaluate(expr)
      print "-_-_-_-_-_-_"

def test_suite():
    return unittest.makeSuite(XPathTests)

if __name__=='__main__':
    main()
